@@ -66,6 +66,7 @@ program other_2d
   character(20), allocatable, dimension(:,:) :: val_w  ! ƥȥǡ
   character(100) :: skip_char    ! Ƭإå̻
                                  ! ɤФԿưեޥåȤκݻ.
+  character(70) :: sysfont
 !-- tmp
   character(80), allocatable, dimension(:) :: dummy_read
   character(80) :: tmp_c
@@ -84,7 +85,7 @@ program other_2d
   namelist /input_2d /flist, dc_val, dz, z_bot,  &
   &                  z_top, IWS, title_txt,  &
   &                  cmin, cmax, smin, smax, cnum, snum, cform, sform,  &
-  &                  undef, skip_num, skip_char, inv_flag, z_title
+  &                  undef, skip_num, skip_char, inv_flag, z_title, sysfont
   read(5,nml=input_2d)
 
   namelist /input_1d /conv_dat, conv_list, undef_1d
@@ -117,8 +118,16 @@ program other_2d
   allocate(hour_v(nt))
 
 !-- DCL set
-  call SGISET('IFONT', 2)
-  call UZFACT(0.65)
+  if(len_trim(adjustl(sysfont))/=0)then
+     call SGISET( 'IFONT', 1 )
+     call SWLSET( 'SYSFONT', .true. )
+     write(*,*) "This drawing mode is sysfont."
+  else
+     call SGISET( 'IFONT', 2 )
+     write(*,*) "This drawing mode is dclfont."
+  end if
+
+  call UZFACT(0.8)
   call DclSetParm( 'ENABLE_CONTOUR_MESSAGE', .false. )
   CALL GLRSET( 'RMISS', undef )
   CALL GLLSET( 'LMISS', .TRUE. )
@@ -281,6 +290,11 @@ program other_2d
 
   call DclOpenGraphics(IWS)
 
+  if(len_trim(adjustl(sysfont))/=0)then
+!     CALL SWSLFT(sysfont)
+     CALL SWCSET('FONTNAME', sysfont)
+  end if
+
   if(v_flag.eqv..false.)then
      if(inv_flag.eqv..false.)then
         call Dcl_2D_Cont_Shade_Calendar( trim(title_txt),  &
