@@ -58,8 +58,9 @@ program sound_1d
   character(5) :: draw_flag  ! ޤ˴ؤե饰
   character(10) :: t_c
   character(28) :: skew_c
-  character(10), allocatable, dimension(:) :: title_c, title_u
+  character(12), allocatable, dimension(:) :: title_c, title_u
   character(1) :: refer_flag
+  character(70) :: sysfont
 
   logical :: dmp_flag, no_draw_flag
 
@@ -69,7 +70,8 @@ program sound_1d
 !-- namelist file ɤ߹
 
   namelist /input /z_ref, p_ref, list_name, dmp_flag, undef, draw_flag,  &
-  &                draw_region_p, draw_region_z, IWS, refer_flag, unit_v
+  &                draw_region_p, draw_region_z, IWS, refer_flag, unit_v,  &
+  &                sysfont
   read(5,nml=input)
 
   if(z_ref==-999.0)then
@@ -95,11 +97,12 @@ program sound_1d
   allocate(cvx(jmax,fig_num))
   allocate(cvy(jmax,fig_num))
 
-  title_c=(/'z_ref"', 'p_ref"', 'CAPE  ', 'CIN   ', 'PW    ', 'z_LCL"',  &
-  &         'z_LFC"', 'z_LNB"', 'p_LCL"', 'p_LFC"', 'p_LNB"'/)
-  title_u=(/' m       ', ' hPa     ', ' J kg|-1"', ' J kg|-1"',  &
-  &         ' mm      ', ' m       ',   &
-  &         ' m       ', ' m       ', ' hPa     ', ' hPa     ', ' hPa     '/)
+  title_c=(/'z\_{ref}', 'p\_{ref}', 'CAPE    ', 'CIN     ', 'PW      ',  &
+  &         'z\_{LCL}', 'z\_{LFC}', 'z\_{LNB}', 'p\_{LCL}', 'p\_{LFC}',  &
+  &         'p\_{LNB}'/)
+  title_u=(/' m         ', ' hPa       ', ' J kg\^{-1}', ' J kg\^{-1}',  &
+  &         ' mm        ', ' m         ', ' m         ', ' m         ',  &
+  &         ' hPa       ', ' hPa       ', ' hPa       '/)
 
   do i=1,fig_num
      cvx(:,i)=convx((i-1)*2+1)+0.01
@@ -122,12 +125,27 @@ program sound_1d
 
 !-- DCL set
   if(no_draw_flag.eqv..false.)then
+     if(len_trim(adjustl(sysfont))/=0)then
+        call SGISET( 'IFONT', 1 )
+        call SWLSET( 'SYSFONT', .true. )
+        write(*,*) "This drawing mode is sysfont."
+     else
+        call SGISET( 'IFONT', 2 )
+        write(*,*) "This drawing mode is dclfont."
+     end if
+
      call DclOpenGraphics(IWS)
+
+     if(len_trim(adjustl(sysfont))/=0)then
+!        CALL SWSLFT(sysfont)
+        CALL SWCSET('FONTNAME', sysfont)
+     end if
+
   end if
 
   call GLRSET('RMISS',undef)
   call GLLSET('LMISS',.true.)
-  call UZFACT( 0.6 )
+  call UZFACT( 0.8 )
   call DclSetParm('ENABLE_PROPORTIONAL_FONT',.true.)
 
   call read_file_text( trim(list_name), 2, nnz, fname )
@@ -299,8 +317,8 @@ program sound_1d
         call Dcl_PL( 'l', trim(fname(2,i)),  &
   &                  draw_yoko(:,1:3), draw_tate(:,1:3),  &
   &                  draw_yoko(:,1:3), draw_tate(:,1:3),  &
-  &                  (/achar(241)//'(black), '//achar(241)//'_e"(red), '//  &
-  &                    achar(241)//'_es"(green)  (K)', 'height (m)'/),  &
+  &                  (/achar(241)//'(black), '//achar(241)//'\_{e}(red), '//  &
+  &                    achar(241)//'\_{es}(green)  (K)', 'height (m)'/),  &
   &                  x_int=(/280.0, 380.0/), y_int=draw_region_z,  &
   &                  viewx_int=(/0.3, 0.7/), viewy_int=(/0.2, 0.8/),  &
   &                  l_idx=(/14,24,34/) )
@@ -427,8 +445,8 @@ program sound_1d
      write(10,'(a'//trim(t_c)//')') trim(tmp_char)
      tmp_char=''
      t_c=''
-     tmp_char="'character' "//"'m' "//"'Pa' "//"'"//'J kg|-1"'//"' "//  &
-  &                   "'"//'J kg|-1"'//"' "//"'mm' "//"'m' "//"'m' "//  &
+     tmp_char="'character' "//"'m' "//"'Pa' "//"'"//'J kg^{-1}'//"' "//  &
+  &                   "'"//'J kg^{-1}'//"' "//"'mm' "//"'m' "//"'m' "//  &
   &                   "'m' "//"'Pa' "//"'Pa' "//"'Pa' " 
      t_c=trim( adjustl( i2c_convert( len_trim(tmp_char) ) ) )
      write(10,'(a'//trim(t_c)//')') trim(tmp_char)
