@@ -67,6 +67,7 @@ program sound_2d
   character(10) :: conv_list     ! ǡե饰
   character(80), allocatable, dimension(:,:) :: fname  ! ե̾
   character(20), allocatable, dimension(:,:) :: val_w  ! ƥȥǡ
+  character(70) :: sysfont
 !-- tmp
   character(80) :: tmp_c
 
@@ -82,7 +83,7 @@ program sound_2d
   namelist /input /flist, conv_dat, conv_list, conv_undef, dz, z_bot,  &
   &                z_top, IWS, title_txt,  &
   &                cmin, cmax, smin, smax, cont_val, shade_val,  &
-  &                vec_val, cnum, snum, undef
+  &                vec_val, cnum, snum, undef, sysfont
   read(5,nml=input)
 
   fflag=.false.
@@ -104,8 +105,16 @@ program sound_2d
   allocate(hour_v(nt))
 
 !-- DCL set
-  call SGISET('IFONT', 2)
-  call UZFACT(0.6)
+  if(len_trim(adjustl(sysfont))/=0)then
+     call SGISET( 'IFONT', 1 )
+     call SWLSET( 'SYSFONT', .true. )
+     write(*,*) "This drawing mode is sysfont."
+  else
+     call SGISET( 'IFONT', 2 )
+     write(*,*) "This drawing mode is dclfont."
+  end if
+
+  call UZFACT(0.8)
   call DclSetParm( 'ENABLE_CONTOUR_MESSAGE', .false. )
   CALL GLRSET( 'RMISS', undef )
   CALL GLLSET( 'LMISS', .TRUE. )
@@ -302,6 +311,11 @@ program sound_2d
 
   call DclOpenGraphics(IWS)
 
+  if(len_trim(adjustl(sysfont))/=0)then
+!     CALL SWSLFT(sysfont)
+     CALL SWCSET('FONTNAME', sysfont)
+  end if
+
   if(vec_val.eqv..false.)then
      call Dcl_2D_Cont_Shade_Calendar( trim(title_txt),  &
   &       time, z, val_draw(cont_flag,:,:), val_draw(shade_flag,:,:),  &
