@@ -17,10 +17,11 @@ program sound_conv
   character(200), allocatable, dimension(:) :: iname, oname
   character(10) :: unity(6)  ! unit convert flags
   character(20) :: foma, fomb
-  real :: conv_undef
+  character(1) :: hydro_flag
+  real :: conv_undef, hydro_ref
   real :: limit_height, factor, ew_tmp, ww_tmp, snd_height, conv_height, dz_conv
   real, allocatable, dimension(:) :: height, pres, temp, vapor, tmp, ew, ww
-  logical :: limit_flag, conv_inter, continu_flag, write_flag
+  logical :: limit_flag, conv_inter, continu_flag, write_flag, rev_flag
 
   i_undef=len_trim(undef)
   foma='(6a15)'
@@ -31,7 +32,7 @@ program sound_conv
 !  call read_namelist()
   namelist /input /fname, undef, sign_flag, skip_num, unity, limit_height,  &
   &                snd_height, output_name, dz_conv, conv_inter, conv_undef,  &
-  &                continu_flag
+  &                continu_flag, hydro_ref, rev_flag
   read(5,nml=input)
 
 !-- list file read
@@ -41,6 +42,42 @@ program sound_conv
   allocate(oname(nf))
   call read_file_text( trim(fname), 1, nf, iname )
 
+!-- interpolting data using hydrostatic relation
+
+  hydro_flag='y'
+  do i=1,len_trim(adjustl(sign_flag))
+     if(sign_flag(i:i)=='1')then
+        if(hydro_flag(1:1)=='y')then
+           hydro_flag='p'
+        else if(hydro_flag(1:1)=='z')then
+           hydro_flag='n'
+        end if
+     end if
+     if(sign_flag(i:i)=='3')then
+        if(hydro_flag(1:1)=='y')then
+           hydro_flag='z'
+        else if(hydro_flag(1:1)=='p')then
+           hydro_flag='n'
+        end if
+     end if
+  end do
+
+  if(hydro_flag(1:1)=='z')then
+     write(*,*) "*** MESSAGE (main) *** : hydrostatic interpolation is active."
+     write(*,*) "interpolating for z."
+  else if(hydro_flag(1:1)=='p')then
+     write(*,*) "*** MESSAGE (main) *** : hydrostatic interpolation is active."
+     write(*,*) "interpolating for p."
+  end if
+
+  if(rev_flag.eqv..true.)then
+     write(*,*) "*** MESSAGE (main) *** : rev_flag is .true."
+     write(*,*) "data is reversing (CReSS sounding type)."
+  else
+     write(*,*) "*** MESSAGE (main) *** : rev_flag is .false."
+     write(*,*) "data is not reversing (radio sonde type)."
+  end if
+
 !-- do loop counter
 
   do id=1,nf
@@ -75,11 +112,20 @@ program sound_conv
      do j=1,nx
         if(sign_flag(j:j)/='0')then
            do k=1,nz
-              tmpc=trim(cval(j,k))
+              if(rev_flag.eqv..false.)then
+                 tmpc=trim(cval(j,k))
+              else
+                 tmpc=trim(cval(j,nz-k+1))
+              end if
+
               if(tmpc(1:len_trim(undef))==trim(undef))then
                  val(j,k)=conv_undef
               else
-                 val(j,k)=c2r_convert( cval(j,k) )
+                 if(rev_flag.eqv..false.)then
+                    val(j,k)=c2r_convert( cval(j,k) )
+                 else
+                    val(j,k)=c2r_convert( cval(j,nz-k+1) )
+                 end if
               end if
            end do
         end if
@@ -124,10 +170,10 @@ program sound_conv
               limit_flag=.true.
            end if
         case('2')
-           if(unity(2)=='K')then
-              factor=0.0
-           else
+           if(unity(2)=='degC')then
               factor=273.15
+           else
+              factor=0.0
            end if
            do j=neg_i,nz
               if(val(i,j)/=conv_undef)then
@@ -198,6 +244,52 @@ program sound_conv
         end select
      end do
 
+!-- interpolating z or p using hydrostatic relation
+
+     select case (hydro_flag(1:1))
+     case ('p')
+!        pres(neg_i)=exner_func_dry( hydro_ref )
+        pres(neg_i)=hydro_ref
+        do j=neg_i+1,nz
+           if(height(j)/=conv_undef.and.height(j-1)/=conv_undef)then  ! Temp.
+              if(unity(2)=='K'.or.unity(2)=='degC')then
+                 pres(j)=pres(j-1)*exp(-0.5*(g/Rd)*((height(j)-height(j-1))  &
+  &                                    *(1.0/temp(j-1)+1.0/temp(j))))
+              else if(unity(2)=='PK')then  ! Theta
+                 pres(j)=pres(j-1)*exp((Cpd/Rd)  &
+  &                               *log(1.0-0.5*(g/Cpd)*(height(j)-height(j-1))  &
+  &                                    *(1.0/temp(j-1)+1.0/temp(j))))
+              end if
+           end if
+        end do
+
+     case ('z')
+        height(neg_i)=hydro_ref
+        do j=neg_i+1,nz
+           if(pres(j)/=conv_undef.and.pres(j-1)/=conv_undef)then
+              if(unity(2)=='K'.or.unity(2)=='degC')then
+                 height(j)=height(j-1)-0.5*(Rd/g)*(log(pres(j)/pres(j-1)))  &
+  &                                    *(temp(j)+temp(j-1))
+              else if(unity(2)=='PK')then  ! Theta
+                 height(j)=height(j-1)-0.5*(Cpd/g)*(temp(j)+temp(j-1))*  &
+  &                                    (exner_func_dry( pres(j) )  &
+  &                                    -exner_func_dry( pres(j-1) ))
+              end if
+           end if
+        end do
+
+     end select
+
+!-- convert theta to temp
+
+     if(unity(2)=='PK')then
+        do j=neg_i,nz
+           if(temp(j)/=conv_undef)then
+              temp(j)=thetaP_2_T( temp(j), pres(j) )
+           end if
+        end do
+     end if
+
 !-- convert check RH
 
      if(unity(4)=='%')then
@@ -243,7 +335,6 @@ program sound_conv
         end do
      end if
 
-
 !-- writing file
 
      write(*,*) "--- writing Now: ", trim(oname(id)), " ---"
