@@ -4,9 +4,9 @@
 module Map_Function
 !  Ͽ޺ɸϤˤ뤤餫Ѵؿ
 
-interface rt2ll
-  module procedure rt2ll_f, rt2ll_d
-end interface rt2ll
+interface ll2rt
+  module procedure ll2rt_f, ll2rt_d
+end interface ll2rt
 
 interface ll2radi
   module procedure ll2radi_f, ll2radi_d
@@ -32,14 +32,132 @@ interface lonlat2lamdis
   module procedure lonlat2lamdis_f, lonlat2lamdis_d
 end interface lonlat2lamdis
 
+interface rt2ll
+  module procedure rt2ll_f, rt2ll_d
+end interface rt2ll
+
 contains
 
+subroutine ll2rt_f( lon0, lat0, lon1, lat1, r, theta )
+! ϵε̺ɸϤˤ, (lon0, lat0) ΰ֤Ȥ (r,\theta) ɸ
+! ŸȤ, (lon1, lat1) ΰ֤ˤΥ r, Ʊ̳ theta 
+! ׻. Υ r ϵ̾α߸̵ΥȤ, theta = 90, 270 deg ̤
+! Ҹ̤̾褦 theta ֤.
+! rt2ll εմؿ, Υ ll2radi ѤФ褤.
+! ϿƤȤƤ, ̿ˡ˳.
+  use Phys_Const
+  use Math_Const
+  implicit none
+  real, intent(in) :: lon0          ! ˺ɸη [rad]
+  real, intent(in) :: lat0          ! ˺ɸΰ [rad]
+  real, intent(in) :: lon1          ! ׻줿 [rad]
+  real, intent(in) :: lat1          ! ׻줿 [rad]
+  real, intent(inout) :: r          ! lon0 α߸̵Υ [m]
+  real, intent(inout) :: theta      ! lon0 ǤƱ̳ [rad]
+  real :: tmpcos, tmpsin, tmptan
+
+  r=ll2radi_f( lon0, lat0, lon1, lat1 )
+
+  if(r>0.0)then
+     tmpcos=cos(lat1)*sin(lon1-lon0)
+     tmpsin=sin(lat1)*cos(lat0)-cos(lat1)*sin(lat0)*cos(lon1-lon0)
+     if(tmpcos==0.0.and.tmpsin==0.0)then
+        theta=0.0
+     else if(tmpcos==0.0.and.tmpsin>0.0)then
+        theta=0.5*pi
+     else if(tmpcos==0.0.and.tmpsin<0.0)then
+        theta=1.5*pi
+     else if(tmpcos>0.0.and.tmpsin==0.0)then
+        theta=0.0
+     else if(tmpcos<0.0.and.tmpsin==0.0)then
+        theta=pi
+     else
+        if(tmpcos>0.0.and.tmpsin>0.0)then
+           tmptan=tmpsin/tmpcos
+           theta=atan(tmptan)
+        else if(tmpcos<0.0.and.tmpsin>0.0)then
+           tmptan=tmpsin/abs(tmpcos)
+           theta=pi-atan(tmptan)
+        else if(tmpcos>0.0.and.tmpsin<0.0)then
+           tmptan=abs(tmpsin)/tmpcos
+           theta=2.0*pi-atan(tmptan)
+        else if(tmpcos<0.0.and.tmpsin<0.0)then
+           tmptan=tmpsin/tmpcos
+           theta=pi+atan(tmptan)
+        end if
+     end if
+  else
+     theta=0.0
+  end if
+
+end subroutine ll2rt_f
+
+!-----------------------------------------------
+!-----------------------------------------------
+
+subroutine ll2rt_d( lon0, lat0, lon1, lat1, r, theta )
+! ϵε̺ɸϤˤ, (lon0, lat0) ΰ֤Ȥ (r,\theta) ɸ
+! ŸȤ, (lon1, lat1) ΰ֤ˤΥ r, Ʊ̳ theta 
+! ׻. Υ r ϵ̾α߸̵ΥȤ, theta = 90, 270 deg ̤
+! Ҹ̤̾褦 theta ֤.
+! rt2ll εմؿ, Υ ll2radi ѤФ褤.
+! ϿƤȤƤ, ̿ˡ˳.
+  use Phys_Const
+  use Math_Const
+  implicit none
+  double precision, intent(in) :: lon0          ! ˺ɸη [rad]
+  double precision, intent(in) :: lat0          ! ˺ɸΰ [rad]
+  double precision, intent(in) :: lon1          ! ׻줿 [rad]
+  double precision, intent(in) :: lat1          ! ׻줿 [rad]
+  double precision, intent(inout) :: r          ! lon0 α߸̵Υ [m]
+  double precision, intent(inout) :: theta      ! lon0 ǤƱ̳ [rad]
+  double precision :: tmpcos, tmpsin, tmptan
+
+  r=ll2radi_d( lon0, lat0, lon1, lat1 )
+
+  if(r>0.0d0)then
+     tmpcos=dcos(lat1)*dsin(lon1-lon0)
+     tmpsin=dsin(lat1)*dcos(lat0)-dcos(lat1)*dsin(lat0)*dcos(lon1-lon0)
+     if(tmpcos==0.0d0.and.tmpsin==0.0d0)then
+        theta=0.0d0
+     else if(tmpcos==0.0d0.and.tmpsin>0.0d0)then
+        theta=0.5d0*pi_dp
+     else if(tmpcos==0.0d0.and.tmpsin<0.0d0)then
+        theta=1.5d0*pi_dp
+     else if(tmpcos>0.0d0.and.tmpsin==0.0d0)then
+        theta=0.0d0
+     else if(tmpcos<0.0d0.and.tmpsin==0.0d0)then
+        theta=pi_dp
+     else
+        if(tmpcos>0.0d0.and.tmpsin>0.0d0)then
+           tmptan=tmpsin/tmpcos
+           theta=datan(tmptan)
+        else if(tmpcos<0.0d0.and.tmpsin>0.0d0)then
+           tmptan=tmpsin/dabs(tmpcos)
+           theta=pi_dp-datan(tmptan)
+        else if(tmpcos>0.0d0.and.tmpsin<0.0d0)then
+           tmptan=dabs(tmpsin)/tmpcos
+           theta=2.0d0*pi_dp-datan(tmptan)
+        else if(tmpcos<0.0d0.and.tmpsin<0.0d0)then
+           tmptan=tmpsin/tmpcos
+           theta=pi_dp+datan(tmptan)
+        end if
+     end if
+  else
+     theta=0.0d0
+  end if
+
+end subroutine ll2rt_d
+
+!-----------------------------------------------
+!-----------------------------------------------
+
 subroutine rt2ll_f( r, theta, lon0, lat0, lon, lat )
 ! ϵε̺ɸϤˤ, (lon0, lat0) ΰ֤Ȥ (r,\theta) ɸ
 ! ŸȤ, Υ r, Ʊ̳ theta ΰ֤ˤ̾Ǥη٤
 ! ׻. Υ r ϵ̾α߸̵ΥȤ, theta = 90, 270 deg ̤
 ! Ҹ̤̾褦 theta ֤.
-! ̻ˡѤƷ׻Ԥ,  theta ϵ̻ѷܳ٤ȶ.
+! ϿƤȤƤ, ̿ˡ˳.
   use Phys_Const
   use Math_Const
   implicit none
@@ -56,6 +174,10 @@ subroutine rt2ll_f( r, theta, lon0, lat0, lon, lat )
   latd=180.0*lat0/pi
   rratio=r/radius
 
+  do while(thetad>360.0)
+     thetad=thetad-360.0
+  end do
+
   if(thetad==-90.0.or.thetad==270.0)then
      lon=lon0
      lat=lat0-rratio
@@ -91,7 +213,7 @@ subroutine rt2ll_d( r, theta, lon0, lat0, lon, lat )
 ! ŸȤ, Υ r, Ʊ̳ theta ΰ֤ˤ̾Ǥη٤
 ! ׻. Υ r ϵ̾α߸̵ΥȤ, theta = 90, 270 deg ̤
 ! Ҹ̤̾褦 theta ֤.
-! ̻ˡѤƷ׻Ԥ,  theta ϵ̻ѷܳ٤ȶ.
+! ϿƤȤƤ, ̿ˡ˳.
   use Phys_Const
   use Math_Const
   implicit none
@@ -108,6 +230,10 @@ subroutine rt2ll_d( r, theta, lon0, lat0, lon, lat )
   latd=180.0d0*lat0/pi_dp
   rratio=r/radius
 
+  do while(thetad>360.0d0)
+     thetad=thetad-360.0d0
+  end do
+
   if(thetad==-90.0d0.or.thetad==270.0d0)then
      lon=lon0
      lat=lat0-rratio
