@@ -10,49 +10,49 @@ interface CHEBYSHEV
 
   module procedure CHEBYSHEV_f, CHEBYSHEV_d
 
-end interface
+end interface CHEBYSHEV
 
 interface GEGENBAUER
 
   module procedure GEGENBAUER_f, GEGENBAUER_d
 
-end interface
+end interface GEGENBAUER
 
 interface HERMITE
 
   module procedure HERMITE_f, HERMITE_d
 
-end interface
+end interface HERMITE
 
 interface JACOBI_POLY
 
   module procedure JACOBI_POLY_f, JACOBI_POLY_d
 
-end interface
+end interface JACOBI_POLY
 
 interface LAGUERRE
 
   module procedure LAGUERRE_f, LAGUERRE_d
 
-end interface
+end interface LAGUERRE
 
 interface LEGENDRE
 
   module procedure LEGENDRE_f, LEGENDRE_d
 
-end interface
+end interface LEGENDRE
 
 interface AS_LEGENDRE
 
   module procedure AS_LEGENDRE_f, AS_LEGENDRE_d
 
-end interface
+end interface AS_LEGENDRE
 
 interface SONINE
 
   module procedure SONINE_f, SONINE_d
 
-end interface
+end interface SONINE
 
 
 contains
